<?php
/**
 * Alma settings
 *
 * @package Alma_WooCommerce_Gateway
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Handles settings retrieval from the settings API.
 */
class Alma_WC_Settings {
	const OPTIONS_KEY = 'woocommerce_alma_settings'; // Generated by WooCommerce in WC_Settings_API::get_option_key().
	const AMOUNT_KEYS = array( 'min_amount_2x', 'max_amount_2x', 'min_amount_3x', 'max_amount_3x', 'min_amount_4x', 'max_amount_4x' );

	/**
	 * Setting values from get_option.
	 *
	 * @var array
	 */
	protected $settings = array();

	/**
	 * Flag to indicate setting has been loaded from DB.
	 *
	 * @var bool
	 */
	private $are_settings_loaded = false;

	/**
	 * Get default settings.
	 *
	 * @return array
	 */
	public static function default_settings() {
		return array(
			'enabled'                               => 'yes',
			'enabled_2x'                            => 'no',
			'enabled_3x'                            => 'yes',
			'enabled_4x'                            => 'no',
			'title'                                 => __( 'Monthly Payments with Alma', 'alma-woocommerce-gateway' ),
			'description'                           => __( 'Pay in multiple monthly payments with your credit card.', 'alma-woocommerce-gateway' ),
			'display_cart_eligibility'              => 'yes',
			'display_product_eligibility'           => 'yes',
			'variable_product_price_query_selector' => Alma_WC_Product_Handler::default_variable_price_selector(),
			'excluded_products_list'                => array(),
			'cart_not_eligible_message_gift_cards'  => __( 'Some products cannot be paid with monthly installments', 'alma-woocommerce-gateway' ),
			'live_api_key'                          => '',
			'test_api_key'                          => '',
			'environment'                           => 'test',
			'debug'                                 => 'no',
		);
	}

	/**
	 * __get
	 *
	 * @param string $key Key.
	 *
	 * @return mixed
	 */
	public function __get( $key ) {
		$value = null;

		if ( array_key_exists( $key, $this->settings ) ) {
			$value = $this->settings[ $key ];
		}

		return apply_filters( 'alma_wc_settings_' . $key, $value );
	}

	/**
	 * __set
	 *
	 * @param string $key Key.
	 * @param mixed  $value Value.
	 *
	 * @return void
	 */
	public function __set( $key, $value ) {
		$this->settings[ $key ] = $value;
	}

	/**
	 * __isset
	 *
	 * @param string $key Key.
	 *
	 * @return bool
	 */
	public function __isset( $key ) {
		return array_key_exists( $key, $this->settings );
	}

	/**
	 * __construct
	 *
	 * @return void
	 */
	public function __construct() {
		$this->load();
	}

	/**
	 * Load settings from DB.
	 *
	 * @param bool $force_reload Force reload settings.
	 *
	 * @return Alma_WC_Settings Instance of Alma_Settings
	 */
	public function load( $force_reload = false ) {
		if ( $this->are_settings_loaded && ! $force_reload ) {
			return $this;
		}
		$settings                  = (array) get_option( self::OPTIONS_KEY, array() );
		$this->settings            = array_merge( self::default_settings(), $settings );
		$this->are_settings_loaded = true;

		return $this;
	}

	/**
	 * Update from settings.
	 *
	 * @param mixed $settings Settings.
	 *
	 * @return void
	 */
	public function update_from( $settings = array() ) {
		foreach ( $settings as $key => $value ) {
			$this->settings[ $key ] = $value;
		}
	}

	/**
	 * Save settings.
	 *
	 * @return void
	 */
	public function save() {
		update_option( self::OPTIONS_KEY, $this->settings );
	}

	/**
	 * Get API key for live environment.
	 *
	 * @return string
	 */
	public function get_live_api_key() {
		return $this->live_api_key;
	}

	/**
	 * Get API key for test environment.
	 *
	 * @return string
	 */
	public function get_test_api_key() {
		return $this->test_api_key;
	}

	/**
	 * Get API string for the current environment.
	 *
	 * @return string
	 */
	public function get_active_api_key() {
		return $this->is_live() ? $this->get_live_api_key() : $this->get_test_api_key();
	}

	/**
	 * Need API key.
	 *
	 * @return bool
	 */
	public function need_api_key() {
		return empty( $this->get_active_api_key() );
	}

	/**
	 * Is plugin enabled.
	 *
	 * @return bool
	 */
	public function is_enabled() {
		return 'yes' === $this->enabled;
	}

	/**
	 * Is pnx enabled.
	 *
	 * @param int $installments Number of installments.
	 *
	 * @return bool
	 */
	public function is_pnx_enabled( $installments ) {
		return 'yes' === $this->__get( "enabled_${installments}x" );
	}

	/**
	 * Get enabled pnx plans list.
	 *
	 * @return array
	 */
	public function enabled_pnx_plans() {
		$pnx_list = array();

		foreach ( array( 2, 3, 4 ) as $installments ) {
			if ( $this->is_pnx_enabled( $installments ) ) {
				$pnx_list[] = array(
					'installments_count' => $installments,
					'min_amount'         => $this->get_min_amount( $installments ),
					'max_amount'         => $this->get_max_amount( $installments ),
				);
			}
		}

		return $pnx_list;
	}

	/**
	 * Get min amount for pnx.
	 *
	 * @param int $installments the number of installments.
	 *
	 * @return int
	 */
	public function get_min_amount( $installments ) {
		return $this->__get( "min_amount_${installments}x" );
	}

	/**
	 * Get max amount for pnx.
	 *
	 * @param int $installments the number of installments.
	 *
	 * @return int
	 */
	public function get_max_amount( $installments ) {
		return $this->__get( "max_amount_${installments}x" );
	}

	/**
	 * Get eligible installments for price.
	 *
	 * @param int $amount the amount to pay.
	 *
	 * @return int[]
	 */
	public function get_eligible_installments( $amount ) {
		$enabled_plans         = $this->enabled_pnx_plans();
		$eligible_installments = array();

		foreach ( $enabled_plans as $plan ) {
			if ( $amount >= $plan['min_amount'] && $amount <= $plan['max_amount'] ) {
				$eligible_installments[] = $plan['installments_count'];
			}
		}

		return $eligible_installments;
	}

	/**
	 * Get eligible installments for cart.
	 *
	 * @return int[]
	 */
	public function get_eligible_installments_for_cart() {
		$cart       = new Alma_WC_Cart();
		$cart_total = $cart->get_total();

		return $this->get_eligible_installments( $cart_total );
	}

	/**
	 * Is cart eligible.
	 *
	 * @return bool
	 */
	public function is_cart_eligible() {
		return count( $this->get_eligible_installments_for_cart() ) > 0;
	}

	/**
	 * Get min eligible amount.
	 *
	 * @return int
	 */
	public function get_min_eligible_amount() {
		$enabled_plans = $this->enabled_pnx_plans();

		$min_amount = INF;

		foreach ( $enabled_plans as $plan ) {
			$min_amount = min( $min_amount, $plan['min_amount'] );
		}

		return $min_amount;
	}

	/**
	 * Get max eligible amount.
	 *
	 * @return int
	 */
	public function get_max_eligible_amount() {
		$allowed_plans_list = $this->enabled_pnx_plans();

		$max_amount = 0;

		foreach ( $allowed_plans_list as $plan ) {
			$max_amount = max( $max_amount, $plan['max_amount'] );
		}

		return $max_amount;
	}

	/**
	 * Is plugin "usable", i.e. is it enabled and correctly configured
	 *
	 * @return bool
	 */
	public function is_usable() {
		$user_cant_see = $this->get_environment() === 'test' && ! current_user_can( 'administrator' );

		return $this->is_enabled() && $this->fully_configured && ! $user_cant_see;
	}

	/**
	 * Is logging enabled.
	 *
	 * @return bool
	 */
	public function is_logging_enabled() {
		return 'yes' === $this->debug;
	}

	/**
	 * Get active environment from setting.
	 *
	 * @return string
	 */
	public function get_environment() {
		return 'live' === $this->environment ? 'live' : 'test';
	}

	/**
	 * Is using live API.
	 *
	 * @return bool
	 */
	public function is_live() {
		return $this->get_environment() === 'live';
	}

	/**
	 * Is using test API.
	 *
	 * @return bool
	 */
	public function is_test() {
		return $this->get_environment() === 'test';
	}
}
